# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.shared import GenericDeleteResponse
from scale_gp.types.evaluation_datasets import (
    TestCase,
    TestCaseBatchResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTestCases:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.create(
            evaluation_dataset_id="evaluation_dataset_id",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.create(
            evaluation_dataset_id="evaluation_dataset_id",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
                "expected_extra_info": {
                    "chunks": [
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                    ],
                    "schema_type": "CHUNKS",
                },
                "expected_output": "expected_output",
            },
            account_id="account_id",
            chat_history={},
            test_case_metadata={},
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.test_cases.with_raw_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.evaluation_datasets.test_cases.with_streaming_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.create(
                evaluation_dataset_id="",
                test_case_data={
                    "artifact_ids_filter": ["string", "string", "string"],
                    "input": "input",
                },
            )

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.retrieve(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.test_cases.with_raw_response.retrieve(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.evaluation_datasets.test_cases.with_streaming_response.retrieve(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.retrieve(
                test_case_id="test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.retrieve(
                test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    def test_method_update_overload_1(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_method_update_with_all_params_overload_1(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            chat_history={},
            restore=False,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
                "expected_extra_info": {
                    "chunks": [
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                    ],
                    "schema_type": "CHUNKS",
                },
                "expected_output": "expected_output",
            },
            test_case_metadata={},
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_raw_response_update_overload_1(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.test_cases.with_raw_response.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_streaming_response_update_overload_1(self, client: SGPClient) -> None:
        with client.evaluation_datasets.test_cases.with_streaming_response.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_overload_1(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.update(
                test_case_id="test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.update(
                test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    def test_method_update_overload_2(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            restore=True,
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_raw_response_update_overload_2(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.test_cases.with_raw_response.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            restore=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    def test_streaming_response_update_overload_2(self, client: SGPClient) -> None:
        with client.evaluation_datasets.test_cases.with_streaming_response.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            restore=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_overload_2(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.update(
                test_case_id="test_case_id",
                evaluation_dataset_id="",
                restore=True,
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.update(
                test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
                restore=True,
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(SyncPageResponse[TestCase], test_case, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.list(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            include_archived=True,
            limit=1,
            page=1,
        )
        assert_matches_type(SyncPageResponse[TestCase], test_case, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.test_cases.with_raw_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(SyncPageResponse[TestCase], test_case, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.evaluation_datasets.test_cases.with_streaming_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(SyncPageResponse[TestCase], test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.list(
                evaluation_dataset_id="",
            )

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.delete(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(GenericDeleteResponse, test_case, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.test_cases.with_raw_response.delete(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(GenericDeleteResponse, test_case, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.evaluation_datasets.test_cases.with_streaming_response.delete(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(GenericDeleteResponse, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.delete(
                test_case_id="test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.delete(
                test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    def test_method_batch(self, client: SGPClient) -> None:
        test_case = client.evaluation_datasets.test_cases.batch(
            evaluation_dataset_id="evaluation_dataset_id",
            items=[
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
            ],
        )
        assert_matches_type(TestCaseBatchResponse, test_case, path=["response"])

    @parametrize
    def test_raw_response_batch(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.test_cases.with_raw_response.batch(
            evaluation_dataset_id="evaluation_dataset_id",
            items=[
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(TestCaseBatchResponse, test_case, path=["response"])

    @parametrize
    def test_streaming_response_batch(self, client: SGPClient) -> None:
        with client.evaluation_datasets.test_cases.with_streaming_response.batch(
            evaluation_dataset_id="evaluation_dataset_id",
            items=[
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(TestCaseBatchResponse, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_batch(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.test_cases.with_raw_response.batch(
                evaluation_dataset_id="",
                items=[
                    {
                        "test_case_data": {
                            "artifact_ids_filter": ["string", "string", "string"],
                            "input": "input",
                        }
                    },
                    {
                        "test_case_data": {
                            "artifact_ids_filter": ["string", "string", "string"],
                            "input": "input",
                        }
                    },
                    {
                        "test_case_data": {
                            "artifact_ids_filter": ["string", "string", "string"],
                            "input": "input",
                        }
                    },
                ],
            )


class TestAsyncTestCases:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.create(
            evaluation_dataset_id="evaluation_dataset_id",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.create(
            evaluation_dataset_id="evaluation_dataset_id",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
                "expected_extra_info": {
                    "chunks": [
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                    ],
                    "schema_type": "CHUNKS",
                },
                "expected_output": "expected_output",
            },
            account_id="account_id",
            chat_history={},
            test_case_metadata={},
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.test_cases.with_raw_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.test_cases.with_streaming_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.create(
                evaluation_dataset_id="",
                test_case_data={
                    "artifact_ids_filter": ["string", "string", "string"],
                    "input": "input",
                },
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.retrieve(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.test_cases.with_raw_response.retrieve(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.test_cases.with_streaming_response.retrieve(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.retrieve(
                test_case_id="test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.retrieve(
                test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    async def test_method_update_overload_1(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_method_update_with_all_params_overload_1(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            chat_history={},
            restore=False,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
                "expected_extra_info": {
                    "chunks": [
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                    ],
                    "schema_type": "CHUNKS",
                },
                "expected_output": "expected_output",
            },
            test_case_metadata={},
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_raw_response_update_overload_1(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.test_cases.with_raw_response.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_streaming_response_update_overload_1(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.test_cases.with_streaming_response.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_overload_1(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.update(
                test_case_id="test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.update(
                test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    async def test_method_update_overload_2(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            restore=True,
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_raw_response_update_overload_2(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.test_cases.with_raw_response.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            restore=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @parametrize
    async def test_streaming_response_update_overload_2(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.test_cases.with_streaming_response.update(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            restore=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_overload_2(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.update(
                test_case_id="test_case_id",
                evaluation_dataset_id="",
                restore=True,
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.update(
                test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
                restore=True,
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(AsyncPageResponse[TestCase], test_case, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.list(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            include_archived=True,
            limit=1,
            page=1,
        )
        assert_matches_type(AsyncPageResponse[TestCase], test_case, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.test_cases.with_raw_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(AsyncPageResponse[TestCase], test_case, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.test_cases.with_streaming_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(AsyncPageResponse[TestCase], test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.list(
                evaluation_dataset_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.delete(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(GenericDeleteResponse, test_case, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.test_cases.with_raw_response.delete(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(GenericDeleteResponse, test_case, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.test_cases.with_streaming_response.delete(
            test_case_id="test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(GenericDeleteResponse, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.delete(
                test_case_id="test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.delete(
                test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    async def test_method_batch(self, async_client: AsyncSGPClient) -> None:
        test_case = await async_client.evaluation_datasets.test_cases.batch(
            evaluation_dataset_id="evaluation_dataset_id",
            items=[
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
            ],
        )
        assert_matches_type(TestCaseBatchResponse, test_case, path=["response"])

    @parametrize
    async def test_raw_response_batch(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.test_cases.with_raw_response.batch(
            evaluation_dataset_id="evaluation_dataset_id",
            items=[
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(TestCaseBatchResponse, test_case, path=["response"])

    @parametrize
    async def test_streaming_response_batch(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.test_cases.with_streaming_response.batch(
            evaluation_dataset_id="evaluation_dataset_id",
            items=[
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
                {
                    "test_case_data": {
                        "artifact_ids_filter": ["string", "string", "string"],
                        "input": "input",
                    }
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(TestCaseBatchResponse, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_batch(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.test_cases.with_raw_response.batch(
                evaluation_dataset_id="",
                items=[
                    {
                        "test_case_data": {
                            "artifact_ids_filter": ["string", "string", "string"],
                            "input": "input",
                        }
                    },
                    {
                        "test_case_data": {
                            "artifact_ids_filter": ["string", "string", "string"],
                            "input": "input",
                        }
                    },
                    {
                        "test_case_data": {
                            "artifact_ids_filter": ["string", "string", "string"],
                            "input": "input",
                        }
                    },
                ],
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from ..._models import BaseModel

__all__ = ["CancelKnowledgeBaseUploadResponse"]


class CancelKnowledgeBaseUploadResponse(BaseModel):
    canceled: bool
    """Whether cancellation was successful."""

    upload_id: str
    """ID of the knowledge base upload job that was cancelled."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .application_file import ApplicationFile

__all__ = ["ApplicationUploadFilesResponse"]

ApplicationUploadFilesResponse: TypeAlias = List[ApplicationFile]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["GoogleDriveDataSourceConfigParam"]


class GoogleDriveDataSourceConfigParam(TypedDict, total=False):
    drive_id: Required[str]
    """ID associated with the Google Drive to retrieve contents from"""

    source: Required[Literal["GoogleDrive"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List
from typing_extensions import Literal, Required, TypedDict

from .parameter_schema_param import ParameterSchemaParam

__all__ = [
    "ModelTemplateCreateParams",
    "VendorConfiguration",
    "VendorConfigurationBundleConfig",
    "VendorConfigurationEndpointConfig",
    "VendorConfigurationFineTuningJobBundleConfig",
    "VendorConfigurationFineTuningJobBundleConfigResources",
]


class ModelTemplateCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    endpoint_type: Required[Literal["SYNC", "ASYNC", "STREAMING", "BATCH"]]
    """An enum representing the different types of model endpoint types supported.

    Attributes: SYNC: Denotes that the model endpoint type is sync. ASYNC: Denotes
    that the model endpoint type is async. STREAMING: Denotes that the model
    endpoint type is streaming. BATCH: Denotes that the model endpoint type is
    batch.
    """

    model_type: Required[Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"]]
    """An enum representing the different types of models supported.

    Attributes: COMPLETION: Denotes that the model type is completion.
    CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
    that the model type is agent. EMBEDDING: Denotes that the model type is
    embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
    that the model type is generic.
    """

    name: Required[str]

    vendor_configuration: Required[VendorConfiguration]
    """
    Configuration for launching a model using the Launch service which is an
    internal and self-hosted service developed by Scale that deploys models on
    Kubernetes.

    Attributes: vendor: The vendor of the model template bundle_config: The bundle
    configuration of the model template endpoint_config: The endpoint configuration
    of the model template
    """

    endpoint_protocol: Literal["SGP", "COHERE", "VLLM"]
    """An enumeration."""

    model_creation_parameters_schema: ParameterSchemaParam
    """
    The field names and types of available parameter fields which may be specified
    during model creation
    """

    model_request_parameters_schema: ParameterSchemaParam
    """
    The field names and types of available parameter fields which may be specified
    in a model execution API's `model_request_parameters` field.
    """


class VendorConfigurationBundleConfig(TypedDict, total=False):
    image: Required[str]

    registry: Required[str]

    tag: Required[str]

    command: List[str]

    env: Dict[str, str]

    healthcheck_route: str

    predict_route: str

    readiness_initial_delay_seconds: int

    streaming_command: List[str]

    streaming_predict_route: str


class VendorConfigurationEndpointConfig(TypedDict, total=False):
    cpus: int

    endpoint_type: Literal["SYNC", "ASYNC", "STREAMING", "BATCH"]
    """An enum representing the different types of model endpoint types supported.

    Attributes: SYNC: Denotes that the model endpoint type is sync. ASYNC: Denotes
    that the model endpoint type is async. STREAMING: Denotes that the model
    endpoint type is streaming. BATCH: Denotes that the model endpoint type is
    batch.
    """

    gpu_type: Literal[
        "nvidia-tesla-t4",
        "nvidia-ampere-a10",
        "nvidia-ampere-a100",
        "nvidia-ampere-a100e",
        "nvidia-hopper-h100",
        "nvidia-hopper-h100-1g20gb",
        "nvidia-hopper-h100-3g40gb",
    ]
    """An enumeration."""

    gpus: int

    high_priority: bool

    max_workers: int

    memory: str

    min_workers: int

    per_worker: int
    """The maximum number of concurrent requests that an individual worker can service.

    Launch automatically scales the number of workers for the endpoint so that each
    worker is processing `per_worker` requests, subject to the limits defined by
    `min_workers` and `max_workers`.

    - If the average number of concurrent requests per worker is lower than
      `per_worker`, then the number of workers will be reduced. - Otherwise, if the
      average number of concurrent requests per worker is higher than `per_worker`,
      then the number of workers will be increased to meet the elevated traffic.

    Here is our recommendation for computing `per_worker`:

    1. Compute `min_workers` and `max_workers` per your minimum and maximum
       throughput requirements. 2. Determine a value for the maximum number of
       concurrent requests in the workload. Divide this number by `max_workers`.
       Doing this ensures that the number of workers will "climb" to `max_workers`.
    """

    storage: str


class VendorConfigurationFineTuningJobBundleConfigResources(TypedDict, total=False):
    cpus: int

    gpu_type: Literal[
        "nvidia-tesla-t4",
        "nvidia-ampere-a10",
        "nvidia-ampere-a100",
        "nvidia-ampere-a100e",
        "nvidia-hopper-h100",
        "nvidia-hopper-h100-1g20gb",
        "nvidia-hopper-h100-3g40gb",
    ]
    """An enumeration."""

    gpus: int

    memory: str

    storage: str


class VendorConfigurationFineTuningJobBundleConfig(TypedDict, total=False):
    image: Required[str]

    registry: Required[str]

    tag: Required[str]

    command: List[str]

    env: Dict[str, str]

    mount_location: str
    """
    The filesystem location where the fine tuning job's configuration will be
    available when it is started.
    """

    resources: VendorConfigurationFineTuningJobBundleConfigResources

    training_dataset_schema_type: Literal["GENERATION", "RERANKING_QUESTIONS"]
    """An enumeration."""


class VendorConfiguration(TypedDict, total=False):
    bundle_config: Required[VendorConfigurationBundleConfig]

    endpoint_config: VendorConfigurationEndpointConfig

    fine_tuning_job_bundle_config: VendorConfigurationFineTuningJobBundleConfig

    vendor: Literal["LAUNCH"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    studio_project_list_params,
    studio_project_create_params,
    studio_project_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.studio_project import StudioProject
from ..types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["StudioProjectsResource", "AsyncStudioProjectsResource"]


class StudioProjectsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StudioProjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return StudioProjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StudioProjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return StudioProjectsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        description: str,
        name: str,
        studio_api_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StudioProject:
        """
        ### Description

        Creates a studio project

        ### Details

        This API can be used to create a studio project. To use this API, review the
        request schema and pass in all fields that are required to create a studio
        project.

        Args:
          account_id: The ID of the account that owns the given entity.

          description: The description of the Studio Project

          name: The name of the Studio Project

          studio_api_key: Your API key for Studio, can be updated with the PATCH endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/studio-projects",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "description": description,
                    "name": name,
                    "studio_api_key": studio_api_key,
                },
                studio_project_create_params.StudioProjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StudioProject,
        )

    def retrieve(
        self,
        studio_project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StudioProject:
        """
        ### Description

        Gets the details of a studio project

        ### Details

        This API can be used to get information about a single studio project by ID. To
        use this API, pass in the `id` that was returned from your Create Studio Project
        API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not studio_project_id:
            raise ValueError(f"Expected a non-empty value for `studio_project_id` but received {studio_project_id!r}")
        return self._get(
            f"/v4/studio-projects/{studio_project_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StudioProject,
        )

    def update(
        self,
        studio_project_id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        studio_api_key: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StudioProject:
        """
        ### Description

        Updates a studio project

        ### Details

        This API can be used to update the studio project that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Studio Project API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          description: The description of the Studio Project

          name: The name of the Studio Project

          studio_api_key: Your API key for Studio, can be updated with the PATCH endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not studio_project_id:
            raise ValueError(f"Expected a non-empty value for `studio_project_id` but received {studio_project_id!r}")
        return self._patch(
            f"/v4/studio-projects/{studio_project_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "studio_api_key": studio_api_key,
                },
                studio_project_update_params.StudioProjectUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StudioProject,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[StudioProject]:
        """
        ### Description

        Lists all studio projects accessible to the user.

        ### Details

        This API can be used to list studio projects. If a user has access to multiple
        accounts, all studio projects from all accounts the user is associated with will
        be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/studio-projects",
            page=SyncPageResponse[StudioProject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    studio_project_list_params.StudioProjectListParams,
                ),
            ),
            model=StudioProject,
        )

    def delete(
        self,
        studio_project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a studio project

        ### Details

        This API can be used to delete a studio project by ID. To use this API, pass in
        the `id` that was returned from your Create Studio Project API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not studio_project_id:
            raise ValueError(f"Expected a non-empty value for `studio_project_id` but received {studio_project_id!r}")
        return self._delete(
            f"/v4/studio-projects/{studio_project_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncStudioProjectsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStudioProjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStudioProjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStudioProjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncStudioProjectsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        description: str,
        name: str,
        studio_api_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StudioProject:
        """
        ### Description

        Creates a studio project

        ### Details

        This API can be used to create a studio project. To use this API, review the
        request schema and pass in all fields that are required to create a studio
        project.

        Args:
          account_id: The ID of the account that owns the given entity.

          description: The description of the Studio Project

          name: The name of the Studio Project

          studio_api_key: Your API key for Studio, can be updated with the PATCH endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/studio-projects",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "description": description,
                    "name": name,
                    "studio_api_key": studio_api_key,
                },
                studio_project_create_params.StudioProjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StudioProject,
        )

    async def retrieve(
        self,
        studio_project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StudioProject:
        """
        ### Description

        Gets the details of a studio project

        ### Details

        This API can be used to get information about a single studio project by ID. To
        use this API, pass in the `id` that was returned from your Create Studio Project
        API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not studio_project_id:
            raise ValueError(f"Expected a non-empty value for `studio_project_id` but received {studio_project_id!r}")
        return await self._get(
            f"/v4/studio-projects/{studio_project_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StudioProject,
        )

    async def update(
        self,
        studio_project_id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        studio_api_key: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StudioProject:
        """
        ### Description

        Updates a studio project

        ### Details

        This API can be used to update the studio project that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Studio Project API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          description: The description of the Studio Project

          name: The name of the Studio Project

          studio_api_key: Your API key for Studio, can be updated with the PATCH endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not studio_project_id:
            raise ValueError(f"Expected a non-empty value for `studio_project_id` but received {studio_project_id!r}")
        return await self._patch(
            f"/v4/studio-projects/{studio_project_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "studio_api_key": studio_api_key,
                },
                studio_project_update_params.StudioProjectUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StudioProject,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[StudioProject, AsyncPageResponse[StudioProject]]:
        """
        ### Description

        Lists all studio projects accessible to the user.

        ### Details

        This API can be used to list studio projects. If a user has access to multiple
        accounts, all studio projects from all accounts the user is associated with will
        be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/studio-projects",
            page=AsyncPageResponse[StudioProject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    studio_project_list_params.StudioProjectListParams,
                ),
            ),
            model=StudioProject,
        )

    async def delete(
        self,
        studio_project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a studio project

        ### Details

        This API can be used to delete a studio project by ID. To use this API, pass in
        the `id` that was returned from your Create Studio Project API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not studio_project_id:
            raise ValueError(f"Expected a non-empty value for `studio_project_id` but received {studio_project_id!r}")
        return await self._delete(
            f"/v4/studio-projects/{studio_project_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class StudioProjectsResourceWithRawResponse:
    def __init__(self, studio_projects: StudioProjectsResource) -> None:
        self._studio_projects = studio_projects

        self.create = to_raw_response_wrapper(
            studio_projects.create,
        )
        self.retrieve = to_raw_response_wrapper(
            studio_projects.retrieve,
        )
        self.update = to_raw_response_wrapper(
            studio_projects.update,
        )
        self.list = to_raw_response_wrapper(
            studio_projects.list,
        )
        self.delete = to_raw_response_wrapper(
            studio_projects.delete,
        )


class AsyncStudioProjectsResourceWithRawResponse:
    def __init__(self, studio_projects: AsyncStudioProjectsResource) -> None:
        self._studio_projects = studio_projects

        self.create = async_to_raw_response_wrapper(
            studio_projects.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            studio_projects.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            studio_projects.update,
        )
        self.list = async_to_raw_response_wrapper(
            studio_projects.list,
        )
        self.delete = async_to_raw_response_wrapper(
            studio_projects.delete,
        )


class StudioProjectsResourceWithStreamingResponse:
    def __init__(self, studio_projects: StudioProjectsResource) -> None:
        self._studio_projects = studio_projects

        self.create = to_streamed_response_wrapper(
            studio_projects.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            studio_projects.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            studio_projects.update,
        )
        self.list = to_streamed_response_wrapper(
            studio_projects.list,
        )
        self.delete = to_streamed_response_wrapper(
            studio_projects.delete,
        )


class AsyncStudioProjectsResourceWithStreamingResponse:
    def __init__(self, studio_projects: AsyncStudioProjectsResource) -> None:
        self._studio_projects = studio_projects

        self.create = async_to_streamed_response_wrapper(
            studio_projects.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            studio_projects.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            studio_projects.update,
        )
        self.list = async_to_streamed_response_wrapper(
            studio_projects.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            studio_projects.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["ChatThreadFeedback"]


class ChatThreadFeedback(BaseModel):
    id: str

    application_interaction_id: str

    chat_thread_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    description: str

    sentiment: Literal["positive", "negative"]
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .._models import BaseModel

__all__ = ["EvaluationMetrics"]


class EvaluationMetrics(BaseModel):
    avg_labeling_time_per_test_case: float
    """Average time spent labeling per test case in seconds."""

    avg_num_test_cases_labeled_per_day: float
    """Average number of test cases labeled per day."""

    evaluation_id: str
    """The ID of the evaluation."""

    num_test_cases_approved: int
    """Number of test cases approved."""

    num_test_cases_fixed: int
    """Number of test cases fixed."""

    num_test_cases_flagged: int
    """Number of test cases flagged."""

    num_test_cases_labeled: int
    """Number of test cases labeled."""

    num_test_cases_unaudited: int
    """Number of test cases unaudited."""

    num_total_test_cases: int
    """Total number of test cases."""

    percentage_test_cases_approved: float
    """Percentage of test cases that were approved."""

    percentage_test_cases_fixed: float
    """Percentage of test cases that were fixed."""

    percentage_test_cases_unaudited: float
    """Percentage of test cases that were unaudited."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp._utils import parse_datetime
from scale_gp.types.shared import ModelUsage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsageStatistics:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        usage_statistic = client.model_groups.usage_statistics.retrieve(
            model_group_id="model_group_id",
            chunks=0,
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ModelUsage, usage_statistic, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.model_groups.usage_statistics.with_raw_response.retrieve(
            model_group_id="model_group_id",
            chunks=0,
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage_statistic = response.parse()
        assert_matches_type(ModelUsage, usage_statistic, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.model_groups.usage_statistics.with_streaming_response.retrieve(
            model_group_id="model_group_id",
            chunks=0,
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage_statistic = response.parse()
            assert_matches_type(ModelUsage, usage_statistic, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_group_id` but received ''"):
            client.model_groups.usage_statistics.with_raw_response.retrieve(
                model_group_id="",
                chunks=0,
                end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )


class TestAsyncUsageStatistics:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        usage_statistic = await async_client.model_groups.usage_statistics.retrieve(
            model_group_id="model_group_id",
            chunks=0,
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ModelUsage, usage_statistic, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.model_groups.usage_statistics.with_raw_response.retrieve(
            model_group_id="model_group_id",
            chunks=0,
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage_statistic = await response.parse()
        assert_matches_type(ModelUsage, usage_statistic, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.model_groups.usage_statistics.with_streaming_response.retrieve(
            model_group_id="model_group_id",
            chunks=0,
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage_statistic = await response.parse()
            assert_matches_type(ModelUsage, usage_statistic, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_group_id` but received ''"):
            await async_client.model_groups.usage_statistics.with_raw_response.retrieve(
                model_group_id="",
                chunks=0,
                end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ContributorMetrics"]


class ContributorMetrics(BaseModel):
    annotated_by_user_id: str
    """The ID of the user who annotated the test case."""

    evaluation_id: str
    """The ID of the evaluation."""

    num_test_cases_fixed: int
    """Number of test cases that were fixed."""

    percentage_test_cases_fixed: float
    """Percentage of test cases done by this contributor that were fixed."""

    total_num_test_cases_labeled: int
    """Total number of test cases labeled."""

    avg_time_spent_labeling_sec: Optional[float] = None
    """Average time spent labeling per test case in seconds."""

    total_time_spent_labeling_sec: Optional[int] = None
    """Total time spent labeling in seconds."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .task import Task as Task
from .theme import Theme as Theme
from .shared import (
    ModelUsage as ModelUsage,
    CompletionResponse as CompletionResponse,
    ChunkExtraInfoSchema as ChunkExtraInfoSchema,
    GenericDeleteResponse as GenericDeleteResponse,
    StringExtraInfoSchema as StringExtraInfoSchema,
    ResultSchemaGeneration as ResultSchemaGeneration,
)
from .question import Question as Question
from .dashboard import Dashboard as Dashboard
from .user_info import UserInfo as UserInfo
from .evaluation import Evaluation as Evaluation
from .chat_thread import ChatThread as ChatThread
from .model_group import ModelGroup as ModelGroup
from .scalar_data import ScalarData as ScalarData
from .question_set import QuestionSet as QuestionSet
from .knowledge_base import KnowledgeBase as KnowledgeBase
from .model_instance import ModelInstance as ModelInstance
from .model_template import ModelTemplate as ModelTemplate
from .studio_project import StudioProject as StudioProject
from .chunks_response import ChunksResponse as ChunksResponse
from .fine_tuning_job import FineTuningJob as FineTuningJob
from .timeseries_data import TimeseriesData as TimeseriesData
from .application_edge import ApplicationEdge as ApplicationEdge
from .application_file import ApplicationFile as ApplicationFile
from .application_node import ApplicationNode as ApplicationNode
from .application_spec import ApplicationSpec as ApplicationSpec
from .paginated_themes import PaginatedThemes as PaginatedThemes
from .parameter_schema import ParameterSchema as ParameterSchema
from .training_dataset import TrainingDataset as TrainingDataset
from .annotation_config import AnnotationConfig as AnnotationConfig
from .chunk_rank_params import ChunkRankParams as ChunkRankParams
from .evaluation_config import EvaluationConfig as EvaluationConfig
from .model_list_params import ModelListParams as ModelListParams
from .theme_list_params import ThemeListParams as ThemeListParams
from .embedding_response import EmbeddingResponse as EmbeddingResponse
from .evaluation_dataset import EvaluationDataset as EvaluationDataset
from .evaluation_metrics import EvaluationMetrics as EvaluationMetrics
from .reranking_response import RerankingResponse as RerankingResponse
from .model_create_params import ModelCreateParams as ModelCreateParams
from .model_update_params import ModelUpdateParams as ModelUpdateParams
from .paginated_questions import PaginatedQuestions as PaginatedQuestions
from .theme_create_params import ThemeCreateParams as ThemeCreateParams
from .agent_execute_params import AgentExecuteParams as AgentExecuteParams
from .completions_response import CompletionsResponse as CompletionsResponse
from .question_list_params import QuestionListParams as QuestionListParams
from .account_create_params import AccountCreateParams as AccountCreateParams
from .evaluation_trace_span import EvaluationTraceSpan as EvaluationTraceSpan
from .evaluation_with_views import EvaluationWithViews as EvaluationWithViews
from .fine_tuning_job_event import FineTuningJobEvent as FineTuningJobEvent
from .model_retrieve_params import ModelRetrieveParams as ModelRetrieveParams
from .paginated_evaluations import PaginatedEvaluations as PaginatedEvaluations
from .s3_data_source_config import S3DataSourceConfig as S3DataSourceConfig
from .application_deployment import ApplicationDeployment as ApplicationDeployment
from .application_edge_param import ApplicationEdgeParam as ApplicationEdgeParam
from .application_node_param import ApplicationNodeParam as ApplicationNodeParam
from .chunk_synthesis_params import ChunkSynthesisParams as ChunkSynthesisParams
from .deployment_package_job import DeploymentPackageJob as DeploymentPackageJob
from .evaluation_list_params import EvaluationListParams as EvaluationListParams
from .execute_agent_response import ExecuteAgentResponse as ExecuteAgentResponse
from .generic_model_response import GenericModelResponse as GenericModelResponse
from .paginated_model_groups import PaginatedModelGroups as PaginatedModelGroups
from .parameter_schema_field import ParameterSchemaField as ParameterSchemaField
from .parameter_schema_param import ParameterSchemaParam as ParameterSchemaParam
from .question_create_params import QuestionCreateParams as QuestionCreateParams
from .ranked_chunks_response import RankedChunksResponse as RankedChunksResponse
from .user_retrieve_response import UserRetrieveResponse as UserRetrieveResponse
from .annotation_config_param import AnnotationConfigParam as AnnotationConfigParam
from .create_account_response import CreateAccountResponse as CreateAccountResponse
from .model_group_list_params import ModelGroupListParams as ModelGroupListParams
from .paginated_question_sets import PaginatedQuestionSets as PaginatedQuestionSets
from .application_metric_score import ApplicationMetricScore as ApplicationMetricScore
from .completion_create_params import CompletionCreateParams as CompletionCreateParams
from .evaluation_create_params import EvaluationCreateParams as EvaluationCreateParams
from .evaluation_update_params import EvaluationUpdateParams as EvaluationUpdateParams
from .local_file_source_config import LocalFileSourceConfig as LocalFileSourceConfig
from .parameter_bindings_param import ParameterBindingsParam as ParameterBindingsParam
from .question_set_list_params import QuestionSetListParams as QuestionSetListParams
from .slack_data_source_config import SlackDataSourceConfig as SlackDataSourceConfig
from .application_configuration import ApplicationConfiguration as ApplicationConfiguration
from .chat_completions_response import ChatCompletionsResponse as ChatCompletionsResponse
from .chat_thread_update_params import ChatThreadUpdateParams as ChatThreadUpdateParams
from .hybrid_evaluation_metrics import HybridEvaluationMetrics as HybridEvaluationMetrics
from .interaction_create_params import InteractionCreateParams as InteractionCreateParams
from .model_group_create_params import ModelGroupCreateParams as ModelGroupCreateParams
from .model_group_update_params import ModelGroupUpdateParams as ModelGroupUpdateParams
from .model_instance_with_views import ModelInstanceWithViews as ModelInstanceWithViews
from .paginated_knowledge_bases import PaginatedKnowledgeBases as PaginatedKnowledgeBases
from .paginated_model_templates import PaginatedModelTemplates as PaginatedModelTemplates
from .paginated_studio_projects import PaginatedStudioProjects as PaginatedStudioProjects
from .application_process_params import ApplicationProcessParams as ApplicationProcessParams
from .evaluation_retrieve_params import EvaluationRetrieveParams as EvaluationRetrieveParams
from .knowledge_base_data_source import KnowledgeBaseDataSource as KnowledgeBaseDataSource
from .knowledge_base_list_params import KnowledgeBaseListParams as KnowledgeBaseListParams
from .local_chunks_source_config import LocalChunksSourceConfig as LocalChunksSourceConfig
from .model_template_list_params import ModelTemplateListParams as ModelTemplateListParams
from .paginated_fine_tuning_jobs import PaginatedFineTuningJobs as PaginatedFineTuningJobs
from .question_set_create_params import QuestionSetCreateParams as QuestionSetCreateParams
from .question_set_list_response import QuestionSetListResponse as QuestionSetListResponse
from .studio_project_list_params import StudioProjectListParams as StudioProjectListParams
from .synthesize_chunks_response import SynthesizeChunksResponse as SynthesizeChunksResponse
from .application_validate_params import ApplicationValidateParams as ApplicationValidateParams
from .fine_tuning_job_list_params import FineTuningJobListParams as FineTuningJobListParams
from .interaction_create_response import InteractionCreateResponse as InteractionCreateResponse
from .knowledge_base_query_params import KnowledgeBaseQueryParams as KnowledgeBaseQueryParams
from .paginated_application_specs import PaginatedApplicationSpecs as PaginatedApplicationSpecs
from .paginated_training_datasets import PaginatedTrainingDatasets as PaginatedTrainingDatasets
from .question_set_with_questions import QuestionSetWithQuestions as QuestionSetWithQuestions
from .s3_data_source_config_param import S3DataSourceConfigParam as S3DataSourceConfigParam
from .application_spec_list_params import ApplicationSpecListParams as ApplicationSpecListParams
from .application_test_case_output import ApplicationTestCaseOutput as ApplicationTestCaseOutput
from .knowledge_base_create_params import KnowledgeBaseCreateParams as KnowledgeBaseCreateParams
from .model_template_create_params import ModelTemplateCreateParams as ModelTemplateCreateParams
from .paginated_evaluation_configs import PaginatedEvaluationConfigs as PaginatedEvaluationConfigs
from .parameter_schema_field_param import ParameterSchemaFieldParam as ParameterSchemaFieldParam
from .studio_project_create_params import StudioProjectCreateParams as StudioProjectCreateParams
from .studio_project_update_params import StudioProjectUpdateParams as StudioProjectUpdateParams
from .training_dataset_list_params import TrainingDatasetListParams as TrainingDatasetListParams
from .chat_completion_create_params import ChatCompletionCreateParams as ChatCompletionCreateParams
from .confluence_data_source_config import ConfluenceDataSourceConfig as ConfluenceDataSourceConfig
from .evaluation_config_list_params import EvaluationConfigListParams as EvaluationConfigListParams
from .fine_tuning_job_create_params import FineTuningJobCreateParams as FineTuningJobCreateParams
from .paginated_evaluation_datasets import PaginatedEvaluationDatasets as PaginatedEvaluationDatasets
from .application_spec_create_params import ApplicationSpecCreateParams as ApplicationSpecCreateParams
from .application_spec_update_params import ApplicationSpecUpdateParams as ApplicationSpecUpdateParams
from .create_knowledge_base_response import CreateKnowledgeBaseResponse as CreateKnowledgeBaseResponse
from .delete_knowledge_base_response import DeleteKnowledgeBaseResponse as DeleteKnowledgeBaseResponse
from .evaluation_dataset_list_params import EvaluationDatasetListParams as EvaluationDatasetListParams
from .knowledge_base_retrieve_params import KnowledgeBaseRetrieveParams as KnowledgeBaseRetrieveParams
from .paginated_application_variants import PaginatedApplicationVariants as PaginatedApplicationVariants
from .share_point_data_source_config import SharePointDataSourceConfig as SharePointDataSourceConfig
from .slack_data_source_config_param import SlackDataSourceConfigParam as SlackDataSourceConfigParam
from .token_chunking_strategy_config import TokenChunkingStrategyConfig as TokenChunkingStrategyConfig
from .training_dataset_create_params import TrainingDatasetCreateParams as TrainingDatasetCreateParams
from .application_configuration_param import ApplicationConfigurationParam as ApplicationConfigurationParam
from .application_upload_files_params import ApplicationUploadFilesParams as ApplicationUploadFilesParams
from .application_variant_list_params import ApplicationVariantListParams as ApplicationVariantListParams
from .application_variant_with_scores import ApplicationVariantWithScores as ApplicationVariantWithScores
from .custom_chunking_strategy_config import CustomChunkingStrategyConfig as CustomChunkingStrategyConfig
from .evaluation_config_create_params import EvaluationConfigCreateParams as EvaluationConfigCreateParams
from .google_drive_data_source_config import GoogleDriveDataSourceConfig as GoogleDriveDataSourceConfig
from .evaluation_dataset_create_params import EvaluationDatasetCreateParams as EvaluationDatasetCreateParams
from .evaluation_dataset_update_params import EvaluationDatasetUpdateParams as EvaluationDatasetUpdateParams
from .local_chunks_source_config_param import LocalChunksSourceConfigParam as LocalChunksSourceConfigParam
from .paginated_knowledge_base_uploads import PaginatedKnowledgeBaseUploads as PaginatedKnowledgeBaseUploads
from .s3_data_source_auth_config_param import S3DataSourceAuthConfigParam as S3DataSourceAuthConfigParam
from .training_dataset_generation_item import TrainingDatasetGenerationItem as TrainingDatasetGenerationItem
from .application_thread_process_params import ApplicationThreadProcessParams as ApplicationThreadProcessParams
from .application_upload_files_response import ApplicationUploadFilesResponse as ApplicationUploadFilesResponse
from .application_variant_create_params import ApplicationVariantCreateParams as ApplicationVariantCreateParams
from .application_variant_list_response import ApplicationVariantListResponse as ApplicationVariantListResponse
from .deployment_package_install_params import DeploymentPackageInstallParams as DeploymentPackageInstallParams
from .evaluation_dataset_publish_params import EvaluationDatasetPublishParams as EvaluationDatasetPublishParams
from .multiturn_annotation_config_param import MultiturnAnnotationConfigParam as MultiturnAnnotationConfigParam
from .paginated_application_deployments import PaginatedApplicationDeployments as PaginatedApplicationDeployments
from .application_deployment_list_params import ApplicationDeploymentListParams as ApplicationDeploymentListParams
from .application_schema_retrieve_params import ApplicationSchemaRetrieveParams as ApplicationSchemaRetrieveParams
from .application_variant_process_params import ApplicationVariantProcessParams as ApplicationVariantProcessParams
from .character_chunking_strategy_config import CharacterChunkingStrategyConfig as CharacterChunkingStrategyConfig
from .knowledge_base_upload_files_params import KnowledgeBaseUploadFilesParams as KnowledgeBaseUploadFilesParams
from .application_variant_create_response import ApplicationVariantCreateResponse as ApplicationVariantCreateResponse
from .confluence_data_source_config_param import ConfluenceDataSourceConfigParam as ConfluenceDataSourceConfigParam
from .install_deployment_package_response import InstallDeploymentPackageResponse as InstallDeploymentPackageResponse
from .paginated_model_instance_with_views import PaginatedModelInstanceWithViews as PaginatedModelInstanceWithViews
from .slack_data_source_auth_config_param import SlackDataSourceAuthConfigParam as SlackDataSourceAuthConfigParam
from .application_deployment_create_params import ApplicationDeploymentCreateParams as ApplicationDeploymentCreateParams
from .application_deployment_update_params import ApplicationDeploymentUpdateParams as ApplicationDeploymentUpdateParams
from .application_schema_retrieve_response import ApplicationSchemaRetrieveResponse as ApplicationSchemaRetrieveResponse
from .share_point_data_source_config_param import SharePointDataSourceConfigParam as SharePointDataSourceConfigParam
from .token_chunking_strategy_config_param import TokenChunkingStrategyConfigParam as TokenChunkingStrategyConfigParam
from .application_variant_retrieve_response import (
    ApplicationVariantRetrieveResponse as ApplicationVariantRetrieveResponse,
)
from .azure_blob_storage_data_source_config import AzureBlobStorageDataSourceConfig as AzureBlobStorageDataSourceConfig
from .custom_chunking_strategy_config_param import (
    CustomChunkingStrategyConfigParam as CustomChunkingStrategyConfigParam,
)
from .google_drive_data_source_config_param import GoogleDriveDataSourceConfigParam as GoogleDriveDataSourceConfigParam
from .paginated_knowledge_base_data_sources import (
    PaginatedKnowledgeBaseDataSources as PaginatedKnowledgeBaseDataSources,
)
from .summarization_annotation_config_param import (
    SummarizationAnnotationConfigParam as SummarizationAnnotationConfigParam,
)
from .application_variant_report_list_params import (
    ApplicationVariantReportListParams as ApplicationVariantReportListParams,
)
from .knowledge_base_data_source_list_params import (
    KnowledgeBaseDataSourceListParams as KnowledgeBaseDataSourceListParams,
)
from .application_node_schema_registry_record import (
    ApplicationNodeSchemaRegistryRecord as ApplicationNodeSchemaRegistryRecord,
)
from .deployment_package_install_async_params import (
    DeploymentPackageInstallAsyncParams as DeploymentPackageInstallAsyncParams,
)
from .evaluation_dataset_approve_batch_params import (
    EvaluationDatasetApproveBatchParams as EvaluationDatasetApproveBatchParams,
)
from .application_test_case_output_list_params import (
    ApplicationTestCaseOutputListParams as ApplicationTestCaseOutputListParams,
)
from .application_variant_report_create_params import (
    ApplicationVariantReportCreateParams as ApplicationVariantReportCreateParams,
)
from .character_chunking_strategy_config_param import (
    CharacterChunkingStrategyConfigParam as CharacterChunkingStrategyConfigParam,
)
from .confluence_data_source_auth_config_param import (
    ConfluenceDataSourceAuthConfigParam as ConfluenceDataSourceAuthConfigParam,
)
from .knowledge_base_data_source_create_params import (
    KnowledgeBaseDataSourceCreateParams as KnowledgeBaseDataSourceCreateParams,
)
from .knowledge_base_data_source_update_params import (
    KnowledgeBaseDataSourceUpdateParams as KnowledgeBaseDataSourceUpdateParams,
)
from .application_test_case_output_batch_params import (
    ApplicationTestCaseOutputBatchParams as ApplicationTestCaseOutputBatchParams,
)
from .application_variant_with_scores_and_views import (
    ApplicationVariantWithScoresAndViews as ApplicationVariantWithScoresAndViews,
)
from .publish_evaluation_dataset_draft_response import (
    PublishEvaluationDatasetDraftResponse as PublishEvaluationDatasetDraftResponse,
)
from .share_point_data_source_auth_config_param import (
    SharePointDataSourceAuthConfigParam as SharePointDataSourceAuthConfigParam,
)
from .application_test_case_output_list_response import (
    ApplicationTestCaseOutputListResponse as ApplicationTestCaseOutputListResponse,
)
from .application_variant_report_retrieve_params import (
    ApplicationVariantReportRetrieveParams as ApplicationVariantReportRetrieveParams,
)
from .google_drive_data_source_auth_config_param import (
    GoogleDriveDataSourceAuthConfigParam as GoogleDriveDataSourceAuthConfigParam,
)
from .application_test_case_output_batch_response import (
    ApplicationTestCaseOutputBatchResponse as ApplicationTestCaseOutputBatchResponse,
)
from .azure_blob_storage_data_source_config_param import (
    AzureBlobStorageDataSourceConfigParam as AzureBlobStorageDataSourceConfigParam,
)
from .application_test_case_output_retrieve_params import (
    ApplicationTestCaseOutputRetrieveParams as ApplicationTestCaseOutputRetrieveParams,
)
from .application_test_case_output_retrieve_response import (
    ApplicationTestCaseOutputRetrieveResponse as ApplicationTestCaseOutputRetrieveResponse,
)
from .azure_blob_storage_data_source_auth_config_param import (
    AzureBlobStorageDataSourceAuthConfigParam as AzureBlobStorageDataSourceAuthConfigParam,
)
from .auto_generated_draft_test_case_missing_chunk_info import (
    AutoGeneratedDraftTestCaseMissingChunkInfo as AutoGeneratedDraftTestCaseMissingChunkInfo,
)
from .create_knowledge_base_uploads_from_files_response import (
    CreateKnowledgeBaseUploadsFromFilesResponse as CreateKnowledgeBaseUploadsFromFilesResponse,
)
from .paginated_application_test_case_output_with_views import (
    PaginatedApplicationTestCaseOutputWithViews as PaginatedApplicationTestCaseOutputWithViews,
)
from .auto_generated_draft_test_case_approve_batch_response import (
    AutoGeneratedDraftTestCaseApproveBatchResponse as AutoGeneratedDraftTestCaseApproveBatchResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["DeploymentPackageJob"]


class DeploymentPackageJob(BaseModel):
    deployment_package_job_id: str

    account_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["CreateKnowledgeBaseUploadsFromFilesResponse"]


class CreateKnowledgeBaseUploadsFromFilesResponse(BaseModel):
    upload_ids: List[str]
    """List of upload IDs that have been kicked off by the upload API"""



import logging
from typing import List, Union, Literal

from ..types import EvaluationDataset
from .._client import SGPClient
from .types.multiturn import MultiturnTestCaseSchema
from .types.summarization import SummarizationTestCaseSchema
from ..types.evaluation_datasets import FlexibleTestCaseSchema, GenerationTestCaseSchema
from ..types.evaluation_datasets.test_case_batch_params import (
    Item as TestCaseItem,
)

log: logging.Logger = logging.getLogger("scale_gp")


TestCaseSchemas = Union[
    FlexibleTestCaseSchema, GenerationTestCaseSchema, SummarizationTestCaseSchema, MultiturnTestCaseSchema
]


class DatasetBuilder:
    def __init__(self, client: SGPClient):
        self.client = client
        self.dataset = None
        self.test_cases = None
        self.dataset_version = None

    def initialize(self, *, account_id: str, name: str, test_cases: List[TestCaseSchemas]) -> EvaluationDataset:
        if len(test_cases) == 0:
            raise ValueError(f"No test cases provided for dataset: {name}")

        formatted_test_cases = self._validate_test_case_schemas(test_cases=test_cases)
        schema_type = self._get_dataset_schema_type(formatted_test_cases)

        dataset = self.client.evaluation_datasets.create(account_id=account_id, schema_type=schema_type, name=name)
        log.info(f"Successfully created dataset with id: {dataset.id}")

        tc_items = [
            TestCaseItem(
                account_id=account_id,
                test_case_data=tc,
            )
            for tc in formatted_test_cases
        ]

        tcs = self.client.evaluation_datasets.test_cases.batch(evaluation_dataset_id=dataset.id, items=tc_items)
        log.info(f"Successfully created {len(tcs)} test cases")

        dataset_version = self.client.evaluation_datasets.evaluation_dataset_versions.create(
            evaluation_dataset_id=dataset.id
        )

        self.dataset = dataset
        self.test_cases = tcs
        self.dataset_version = dataset_version

        return dataset

    def _get_dataset_schema_type(self, test_cases: List[TestCaseSchemas]) -> Literal["FLEXIBLE", "GENERATION"]:
        tc = test_cases[0]

        if isinstance(tc, FlexibleTestCaseSchema):
            return "FLEXIBLE"
        if isinstance(tc, GenerationTestCaseSchema):
            return "GENERATION"

        raise ValueError("Invalid test case schema")

    def _validate_test_case_schemas(self, test_cases: List[TestCaseSchemas]):
        if not test_cases:
            raise ValueError("No test cases provided")

        first_schema_type = type(test_cases[0])

        for i, tc in enumerate(test_cases[1:], start=1):
            if not isinstance(tc, first_schema_type):
                raise ValueError(
                    f"Inconsistent schema types. Test case at index 0 is {first_schema_type.__name__}, "
                    f"but test case at index {i} is {type(tc).__name__}"
                )

        log.info(f"All test cases validated. Schema type: {first_schema_type.__name__}")

        if first_schema_type == SummarizationTestCaseSchema:
            flexible_test_cases = [
                FlexibleTestCaseSchema(
                    input={"document": tc.document, **(tc.other_inputs or {})},
                    expected_output={
                        **({"expected_summary": tc.expected_summary} if tc.expected_summary is not None else {}),
                        **(tc.other_expected or {}),
                    },
                )
                for tc in test_cases
            ]
            return flexible_test_cases
        elif first_schema_type == MultiturnTestCaseSchema:
            flexible_test_cases = [
                FlexibleTestCaseSchema(
                    input={
                        "messages": tc.messages,
                        **({"turns": tc.turns} if tc.turns is not None else {}),
                        **(tc.other_inputs or {}),
                    },
                    expected_output={
                        **({"expected_messages": tc.expected_messages} if tc.expected_messages is not None else {}),
                        **(tc.other_expected or {}),
                    },
                )
                for tc in test_cases
            ]
            return flexible_test_cases

        return test_cases



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["ApplicationSpecUpdateParams", "PartialApplicationSpecRequest", "RestoreRequest"]


class PartialApplicationSpecRequest(TypedDict, total=False):
    description: str
    """The description of the Application Spec"""

    name: str
    """The name of the Application Spec"""

    restore: Literal[False]
    """Set to true to restore the entity from the database."""

    run_online_evaluation: bool
    """Whether the application spec should run online evaluation, default is `false`"""

    theme_id: str


class RestoreRequest(TypedDict, total=False):
    restore: Required[Literal[True]]
    """Set to true to restore the entity from the database."""


ApplicationSpecUpdateParams: TypeAlias = Union[PartialApplicationSpecRequest, RestoreRequest]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncPageResponse, AsyncPageResponse
from ...._base_client import AsyncPaginator, make_request_options
from ....types.models import (
    deployment_list_params,
    deployment_create_params,
    deployment_update_params,
    deployment_execute_params,
    deployment_list_all_params,
)
from .usage_statistics import (
    UsageStatisticsResource,
    AsyncUsageStatisticsResource,
    UsageStatisticsResourceWithRawResponse,
    AsyncUsageStatisticsResourceWithRawResponse,
    UsageStatisticsResourceWithStreamingResponse,
    AsyncUsageStatisticsResourceWithStreamingResponse,
)
from ....types.generic_model_response import GenericModelResponse
from ....types.models.model_deployment import ModelDeployment
from ....types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["DeploymentsResource", "AsyncDeploymentsResource"]


class DeploymentsResource(SyncAPIResource):
    @cached_property
    def usage_statistics(self) -> UsageStatisticsResource:
        return UsageStatisticsResource(self._client)

    @cached_property
    def with_raw_response(self) -> DeploymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return DeploymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DeploymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return DeploymentsResourceWithStreamingResponse(self)

    def create(
        self,
        model_instance_id: str,
        *,
        name: str,
        account_id: str | NotGiven = NOT_GIVEN,
        deployment_metadata: object | NotGiven = NOT_GIVEN,
        model_creation_parameters: object | NotGiven = NOT_GIVEN,
        vendor_configuration: deployment_create_params.VendorConfiguration | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelDeployment:
        """
        ### Description

        Model Deployments are unique endpoints created for custom models in the Scale
        GenAI Platform. They enable users to interact with and utilize specific
        instances of models through the API/SDK. Each deployment is associated with a
        model instance, containing the necessary model template and model-metadata.
        Model templates describe the creation parameters that are configured on the
        deployment. The model deployments provide a means to call upon models for
        inference, logging calls, and monitoring usage.

        Built-in models also have deployments for creating a consistent interface for
        all models. But they don't represent a real deployment, they are just a way to
        interact with the built-in models. These deployments are created automatically
        when the model is created and they are immutable.

        ### Endpoint details

        This endpoint is used to deploy a model instance. The request payload schema
        depends on the `model_request_parameters_schema` of the Model Template that the
        created model was created from.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        return self._post(
            f"/v4/models/{model_instance_id}/deployments",
            body=maybe_transform(
                {
                    "name": name,
                    "account_id": account_id,
                    "deployment_metadata": deployment_metadata,
                    "model_creation_parameters": model_creation_parameters,
                    "vendor_configuration": vendor_configuration,
                },
                deployment_create_params.DeploymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelDeployment,
        )

    def retrieve(
        self,
        deployment_id: str,
        *,
        model_instance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelDeployment:
        """
        ### Description

        Gets the details of a deployment

        ### Details

        This API can be used to get information about a single deployment by ID. To use
        this API, pass in the `id` that was returned from your Create Deployment API
        call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._get(
            f"/v4/models/{model_instance_id}/deployments/{deployment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelDeployment,
        )

    def update(
        self,
        deployment_id: str,
        *,
        model_instance_id: str,
        deployment_metadata: object | NotGiven = NOT_GIVEN,
        model_creation_parameters: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        vendor_configuration: deployment_update_params.VendorConfiguration | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelDeployment:
        """
        ### Description

        Updates a deployment

        ### Details

        This API can be used to update the deployment that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Deployment API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._patch(
            f"/v4/models/{model_instance_id}/deployments/{deployment_id}",
            body=maybe_transform(
                {
                    "deployment_metadata": deployment_metadata,
                    "model_creation_parameters": model_creation_parameters,
                    "name": name,
                    "vendor_configuration": vendor_configuration,
                },
                deployment_update_params.DeploymentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelDeployment,
        )

    def list(
        self,
        model_instance_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort_by: List[
            Literal[
                "model_creation_parameters:asc",
                "model_creation_parameters:desc",
                "model_endpoint_id:asc",
                "model_endpoint_id:desc",
                "model_instance_id:asc",
                "model_instance_id:desc",
                "vendor_configuration:asc",
                "vendor_configuration:desc",
                "deployment_metadata:asc",
                "deployment_metadata:desc",
                "status:asc",
                "status:desc",
                "id:asc",
                "id:desc",
                "created_at:asc",
                "created_at:desc",
                "account_id:asc",
                "account_id:desc",
                "created_by_user_id:asc",
                "created_by_user_id:desc",
                "created_by_user:asc",
                "created_by_user:desc",
                "name:asc",
                "name:desc",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ModelDeployment]:
        """
        TODO: Document

        Args:
          account_id: Optional filter by account id

          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        return self._get_api_list(
            f"/v4/models/{model_instance_id}/deployments",
            page=SyncPageResponse[ModelDeployment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "sort_by": sort_by,
                    },
                    deployment_list_params.DeploymentListParams,
                ),
            ),
            model=ModelDeployment,
        )

    def delete(
        self,
        deployment_id: str,
        *,
        model_instance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a deployment

        ### Details

        This API can be used to delete a deployment by ID. To use this API, pass in the
        `id` that was returned from your Create Deployment API call as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._delete(
            f"/v4/models/{model_instance_id}/deployments/{deployment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    def execute(
        self,
        model_deployment_id: str,
        *,
        model_instance_id: str,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericModelResponse:
        """
        Execute Model Deployment

        Args:
          stream: Flag indicating whether to stream the completion response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        if not model_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `model_deployment_id` but received {model_deployment_id!r}"
            )
        return self._post(
            f"/v4/models/{model_instance_id}/deployments/{model_deployment_id}/execute",
            body=maybe_transform({"stream": stream}, deployment_execute_params.DeploymentExecuteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericModelResponse,
        )

    def list_all(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort_by: List[
            Literal[
                "model_creation_parameters:asc",
                "model_creation_parameters:desc",
                "model_endpoint_id:asc",
                "model_endpoint_id:desc",
                "model_instance_id:asc",
                "model_instance_id:desc",
                "vendor_configuration:asc",
                "vendor_configuration:desc",
                "deployment_metadata:asc",
                "deployment_metadata:desc",
                "status:asc",
                "status:desc",
                "id:asc",
                "id:desc",
                "created_at:asc",
                "created_at:desc",
                "account_id:asc",
                "account_id:desc",
                "created_by_user_id:asc",
                "created_by_user_id:desc",
                "created_by_user:asc",
                "created_by_user:desc",
                "name:asc",
                "name:desc",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ModelDeployment]:
        """
        TODO: Document

        Args:
          account_id: Optional filter by account id

          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/model-deployments",
            page=SyncPageResponse[ModelDeployment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "sort_by": sort_by,
                    },
                    deployment_list_all_params.DeploymentListAllParams,
                ),
            ),
            model=ModelDeployment,
        )


class AsyncDeploymentsResource(AsyncAPIResource):
    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResource:
        return AsyncUsageStatisticsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDeploymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDeploymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDeploymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncDeploymentsResourceWithStreamingResponse(self)

    async def create(
        self,
        model_instance_id: str,
        *,
        name: str,
        account_id: str | NotGiven = NOT_GIVEN,
        deployment_metadata: object | NotGiven = NOT_GIVEN,
        model_creation_parameters: object | NotGiven = NOT_GIVEN,
        vendor_configuration: deployment_create_params.VendorConfiguration | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelDeployment:
        """
        ### Description

        Model Deployments are unique endpoints created for custom models in the Scale
        GenAI Platform. They enable users to interact with and utilize specific
        instances of models through the API/SDK. Each deployment is associated with a
        model instance, containing the necessary model template and model-metadata.
        Model templates describe the creation parameters that are configured on the
        deployment. The model deployments provide a means to call upon models for
        inference, logging calls, and monitoring usage.

        Built-in models also have deployments for creating a consistent interface for
        all models. But they don't represent a real deployment, they are just a way to
        interact with the built-in models. These deployments are created automatically
        when the model is created and they are immutable.

        ### Endpoint details

        This endpoint is used to deploy a model instance. The request payload schema
        depends on the `model_request_parameters_schema` of the Model Template that the
        created model was created from.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        return await self._post(
            f"/v4/models/{model_instance_id}/deployments",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "account_id": account_id,
                    "deployment_metadata": deployment_metadata,
                    "model_creation_parameters": model_creation_parameters,
                    "vendor_configuration": vendor_configuration,
                },
                deployment_create_params.DeploymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelDeployment,
        )

    async def retrieve(
        self,
        deployment_id: str,
        *,
        model_instance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelDeployment:
        """
        ### Description

        Gets the details of a deployment

        ### Details

        This API can be used to get information about a single deployment by ID. To use
        this API, pass in the `id` that was returned from your Create Deployment API
        call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._get(
            f"/v4/models/{model_instance_id}/deployments/{deployment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelDeployment,
        )

    async def update(
        self,
        deployment_id: str,
        *,
        model_instance_id: str,
        deployment_metadata: object | NotGiven = NOT_GIVEN,
        model_creation_parameters: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        vendor_configuration: deployment_update_params.VendorConfiguration | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelDeployment:
        """
        ### Description

        Updates a deployment

        ### Details

        This API can be used to update the deployment that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Deployment API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._patch(
            f"/v4/models/{model_instance_id}/deployments/{deployment_id}",
            body=await async_maybe_transform(
                {
                    "deployment_metadata": deployment_metadata,
                    "model_creation_parameters": model_creation_parameters,
                    "name": name,
                    "vendor_configuration": vendor_configuration,
                },
                deployment_update_params.DeploymentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelDeployment,
        )

    def list(
        self,
        model_instance_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort_by: List[
            Literal[
                "model_creation_parameters:asc",
                "model_creation_parameters:desc",
                "model_endpoint_id:asc",
                "model_endpoint_id:desc",
                "model_instance_id:asc",
                "model_instance_id:desc",
                "vendor_configuration:asc",
                "vendor_configuration:desc",
                "deployment_metadata:asc",
                "deployment_metadata:desc",
                "status:asc",
                "status:desc",
                "id:asc",
                "id:desc",
                "created_at:asc",
                "created_at:desc",
                "account_id:asc",
                "account_id:desc",
                "created_by_user_id:asc",
                "created_by_user_id:desc",
                "created_by_user:asc",
                "created_by_user:desc",
                "name:asc",
                "name:desc",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ModelDeployment, AsyncPageResponse[ModelDeployment]]:
        """
        TODO: Document

        Args:
          account_id: Optional filter by account id

          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        return self._get_api_list(
            f"/v4/models/{model_instance_id}/deployments",
            page=AsyncPageResponse[ModelDeployment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "sort_by": sort_by,
                    },
                    deployment_list_params.DeploymentListParams,
                ),
            ),
            model=ModelDeployment,
        )

    async def delete(
        self,
        deployment_id: str,
        *,
        model_instance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a deployment

        ### Details

        This API can be used to delete a deployment by ID. To use this API, pass in the
        `id` that was returned from your Create Deployment API call as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._delete(
            f"/v4/models/{model_instance_id}/deployments/{deployment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    async def execute(
        self,
        model_deployment_id: str,
        *,
        model_instance_id: str,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericModelResponse:
        """
        Execute Model Deployment

        Args:
          stream: Flag indicating whether to stream the completion response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_instance_id:
            raise ValueError(f"Expected a non-empty value for `model_instance_id` but received {model_instance_id!r}")
        if not model_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `model_deployment_id` but received {model_deployment_id!r}"
            )
        return await self._post(
            f"/v4/models/{model_instance_id}/deployments/{model_deployment_id}/execute",
            body=await async_maybe_transform({"stream": stream}, deployment_execute_params.DeploymentExecuteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericModelResponse,
        )

    def list_all(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort_by: List[
            Literal[
                "model_creation_parameters:asc",
                "model_creation_parameters:desc",
                "model_endpoint_id:asc",
                "model_endpoint_id:desc",
                "model_instance_id:asc",
                "model_instance_id:desc",
                "vendor_configuration:asc",
                "vendor_configuration:desc",
                "deployment_metadata:asc",
                "deployment_metadata:desc",
                "status:asc",
                "status:desc",
                "id:asc",
                "id:desc",
                "created_at:asc",
                "created_at:desc",
                "account_id:asc",
                "account_id:desc",
                "created_by_user_id:asc",
                "created_by_user_id:desc",
                "created_by_user:asc",
                "created_by_user:desc",
                "name:asc",
                "name:desc",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ModelDeployment, AsyncPageResponse[ModelDeployment]]:
        """
        TODO: Document

        Args:
          account_id: Optional filter by account id

          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/model-deployments",
            page=AsyncPageResponse[ModelDeployment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "sort_by": sort_by,
                    },
                    deployment_list_all_params.DeploymentListAllParams,
                ),
            ),
            model=ModelDeployment,
        )


class DeploymentsResourceWithRawResponse:
    def __init__(self, deployments: DeploymentsResource) -> None:
        self._deployments = deployments

        self.create = to_raw_response_wrapper(
            deployments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            deployments.retrieve,
        )
        self.update = to_raw_response_wrapper(
            deployments.update,
        )
        self.list = to_raw_response_wrapper(
            deployments.list,
        )
        self.delete = to_raw_response_wrapper(
            deployments.delete,
        )
        self.execute = to_raw_response_wrapper(
            deployments.execute,
        )
        self.list_all = to_raw_response_wrapper(
            deployments.list_all,
        )

    @cached_property
    def usage_statistics(self) -> UsageStatisticsResourceWithRawResponse:
        return UsageStatisticsResourceWithRawResponse(self._deployments.usage_statistics)


class AsyncDeploymentsResourceWithRawResponse:
    def __init__(self, deployments: AsyncDeploymentsResource) -> None:
        self._deployments = deployments

        self.create = async_to_raw_response_wrapper(
            deployments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            deployments.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            deployments.update,
        )
        self.list = async_to_raw_response_wrapper(
            deployments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            deployments.delete,
        )
        self.execute = async_to_raw_response_wrapper(
            deployments.execute,
        )
        self.list_all = async_to_raw_response_wrapper(
            deployments.list_all,
        )

    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResourceWithRawResponse:
        return AsyncUsageStatisticsResourceWithRawResponse(self._deployments.usage_statistics)


class DeploymentsResourceWithStreamingResponse:
    def __init__(self, deployments: DeploymentsResource) -> None:
        self._deployments = deployments

        self.create = to_streamed_response_wrapper(
            deployments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            deployments.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            deployments.update,
        )
        self.list = to_streamed_response_wrapper(
            deployments.list,
        )
        self.delete = to_streamed_response_wrapper(
            deployments.delete,
        )
        self.execute = to_streamed_response_wrapper(
            deployments.execute,
        )
        self.list_all = to_streamed_response_wrapper(
            deployments.list_all,
        )

    @cached_property
    def usage_statistics(self) -> UsageStatisticsResourceWithStreamingResponse:
        return UsageStatisticsResourceWithStreamingResponse(self._deployments.usage_statistics)


class AsyncDeploymentsResourceWithStreamingResponse:
    def __init__(self, deployments: AsyncDeploymentsResource) -> None:
        self._deployments = deployments

        self.create = async_to_streamed_response_wrapper(
            deployments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            deployments.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            deployments.update,
        )
        self.list = async_to_streamed_response_wrapper(
            deployments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            deployments.delete,
        )
        self.execute = async_to_streamed_response_wrapper(
            deployments.execute,
        )
        self.list_all = async_to_streamed_response_wrapper(
            deployments.list_all,
        )

    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResourceWithStreamingResponse:
        return AsyncUsageStatisticsResourceWithStreamingResponse(self._deployments.usage_statistics)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import theme_list_params, theme_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from ..types.theme import Theme
from .._base_client import AsyncPaginator, make_request_options

__all__ = ["ThemesResource", "AsyncThemesResource"]


class ThemesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ThemesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ThemesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ThemesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ThemesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        logo_blob: str,
        theme_vars: theme_create_params.ThemeVars,
        title: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Theme:
        """
        ### Description

        Custom chat themes

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/themes",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "logo_blob": logo_blob,
                    "theme_vars": theme_vars,
                    "title": title,
                },
                theme_create_params.ThemeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Theme,
        )

    def retrieve(
        self,
        theme_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Theme:
        """
        ### Description

        Gets the details of a theme

        ### Details

        This API can be used to get information about a single theme by ID. To use this
        API, pass in the `id` that was returned from your Create Theme API call as a
        path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not theme_id:
            raise ValueError(f"Expected a non-empty value for `theme_id` but received {theme_id!r}")
        return self._get(
            f"/v4/themes/{theme_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Theme,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[Theme]:
        """
        ### Description

        Lists all themes accessible to the user.

        ### Details

        This API can be used to list themes. If a user has access to multiple accounts,
        all themes from all accounts the user is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/themes",
            page=SyncPageResponse[Theme],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                    },
                    theme_list_params.ThemeListParams,
                ),
            ),
            model=Theme,
        )


class AsyncThemesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncThemesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncThemesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncThemesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncThemesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        logo_blob: str,
        theme_vars: theme_create_params.ThemeVars,
        title: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Theme:
        """
        ### Description

        Custom chat themes

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/themes",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "logo_blob": logo_blob,
                    "theme_vars": theme_vars,
                    "title": title,
                },
                theme_create_params.ThemeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Theme,
        )

    async def retrieve(
        self,
        theme_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Theme:
        """
        ### Description

        Gets the details of a theme

        ### Details

        This API can be used to get information about a single theme by ID. To use this
        API, pass in the `id` that was returned from your Create Theme API call as a
        path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not theme_id:
            raise ValueError(f"Expected a non-empty value for `theme_id` but received {theme_id!r}")
        return await self._get(
            f"/v4/themes/{theme_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Theme,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Theme, AsyncPageResponse[Theme]]:
        """
        ### Description

        Lists all themes accessible to the user.

        ### Details

        This API can be used to list themes. If a user has access to multiple accounts,
        all themes from all accounts the user is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/themes",
            page=AsyncPageResponse[Theme],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                    },
                    theme_list_params.ThemeListParams,
                ),
            ),
            model=Theme,
        )


class ThemesResourceWithRawResponse:
    def __init__(self, themes: ThemesResource) -> None:
        self._themes = themes

        self.create = to_raw_response_wrapper(
            themes.create,
        )
        self.retrieve = to_raw_response_wrapper(
            themes.retrieve,
        )
        self.list = to_raw_response_wrapper(
            themes.list,
        )


class AsyncThemesResourceWithRawResponse:
    def __init__(self, themes: AsyncThemesResource) -> None:
        self._themes = themes

        self.create = async_to_raw_response_wrapper(
            themes.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            themes.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            themes.list,
        )


class ThemesResourceWithStreamingResponse:
    def __init__(self, themes: ThemesResource) -> None:
        self._themes = themes

        self.create = to_streamed_response_wrapper(
            themes.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            themes.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            themes.list,
        )


class AsyncThemesResourceWithStreamingResponse:
    def __init__(self, themes: AsyncThemesResource) -> None:
        self._themes = themes

        self.create = async_to_streamed_response_wrapper(
            themes.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            themes.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            themes.list,
        )



### create_model_example_script.py ###
# This script demonstrates how to deploy and execute a gemini-pro completio model
# 1. Define model template with bundle and endpoint configs
# 2. Create a model instance using the model template
# 3. Create a model deployment using the model instance
# 4. Execute the model deployment to perform a completion
# More information on models can be found at https://egp.dashboard.scale.com/models

import os

from scale_gp import SGPClient
from scale_gp.types.model_template_create_params import (
    VendorConfiguration,
    VendorConfigurationBundleConfig,
    VendorConfigurationEndpointConfig,
)

account_id = os.environ.get("SGP_ACCOUNT_ID", None)
api_key = os.environ.get("SGP_API_KEY", None)

assert (
    account_id is not None
), "You need to set the SGP_ACCOUNT_ID - you can find it at https://egp.dashboard.scale.com/admin/accounts"
assert api_key is not None, "You need to provide your API key - see https://egp.dashboard.scale.com/admin/api-key"

# Create an SGP Client
client = SGPClient(api_key=api_key)

# 1. Define model template
# Before creating a model, you must first create a model template. A model
# template serves 2 purposes. First, it provides common scaffolding that is static
# across multiple models. Second, it exposes several variables that can be
# injected at model creation time to customize the model.
bundle_config = VendorConfigurationBundleConfig(image="gemini-pro", registry="aws-registry", tag="latest")

endpoint_config = VendorConfigurationEndpointConfig(
    max_workers=3,
)

vendor_configuration = VendorConfiguration(
    bundle_config=bundle_config,
    endpoint_config=endpoint_config,
)

model_template = client.model_templates.create(
    account_id=account_id,
    endpoint_type="SYNC",
    model_type="COMPLETION",
    name="Gemini-Pro Template",
    vendor_configuration=vendor_configuration,
)

# 2. Create a model instance using the model template created above
model_instance = client.models.create(
    account_id=account_id,
    model_type="COMPLETION",
    name="gemini-pro",
    model_vendor="GOOGLE",
    model_template_id=model_template.id,
)

# 3. Create a model deployment using the model instance created above
model_deployment = client.models.deployments.create(
    model_instance_id=model_instance.id, name="Gemini-Pro Deployment", account_id=account_id
)

# 4. Execute the model deployment. In this case, we are executing a completion model with a list of prompts.
execute_result = client.models.deployments.execute(
    model_deployment_id=model_deployment.id,
    model_instance_id=model_instance.id,
    extra_body={"prompts": ["What is the capital of Canada?"]},
)

# Print the result
print(execute_result.completions[0][1])
# Returns: ['The capital of Canada is Ottawa. It is located in the province of Ontario, on the south bank of the Ottawa River. The city has a population of over 1 million people and is the fourth largest city in Canada. Ottawa is home to many important government buildings, including Parliament Hill, the Supreme Court of Canada, and the Rideau Hall. It is also a major cultural center, with many museums, art galleries, and theaters.']



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .scalar_retrieve_params import ScalarRetrieveParams as ScalarRetrieveParams
from .timesery_retrieve_params import TimeseryRetrieveParams as TimeseryRetrieveParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from ..auto_generated_draft_test_case_missing_chunk_info import AutoGeneratedDraftTestCaseMissingChunkInfo

__all__ = ["ApproveAutoGeneratedDraftTestCaseResponse"]


class ApproveAutoGeneratedDraftTestCaseResponse(BaseModel):
    autogenerated_draft_test_case_id: str
    """Draft test_case_id that has chunk_ids missing"""

    failed_chunks: List[AutoGeneratedDraftTestCaseMissingChunkInfo]

    success: bool
    """Whether or not the test case was successfully approved"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["UserRetrieveResponse", "AccessProfile", "AccessProfileAccount"]


class AccessProfileAccount(BaseModel):
    id: str

    name: str

    account_group_id: Optional[str] = None
    """The account's parent account group. Optional for backwards compatibility"""


class AccessProfile(BaseModel):
    id: str
    """Access profile id."""

    account: AccessProfileAccount
    """The account in the access profile."""

    role: Literal["manager", "admin", "member", "labeler", "disabled", "invited", "viewer"]
    """An enumeration."""

    user_id: str
    """Id of the user in the access profile."""


class UserRetrieveResponse(BaseModel):
    id: str
    """User id"""

    access_profiles: List[AccessProfile]
    """A list of access profiles that the selected user has access to"""

    email: str
    """E-mail address"""

    first_name: Optional[str] = None
    """First name"""

    is_organization_admin: Optional[bool] = None
    """True if the current user is an organization admin."""

    last_name: Optional[str] = None
    """Last name"""
